<?php
/**
* 2007-2015 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2015 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Paykeeper extends PaymentModule
{
    private $product_cart = array();
    private $order_total = 0;
    private $shipping_price = 0;
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'paykeeper';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.0';
        $this->author = 'paykeeper';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('PayKeeper');
        $this->description = $this->l('PayKeeper payment module');


    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false)
        {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }


        return parent::install() &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('displayPayment') &&
            $this->registerHook('displayPaymentReturn');
    }

    public function uninstall()
    {
        Configuration::deleteByName('PAYKEEPER_URL');
        Configuration::deleteByName('PAYKEEPER_SECRET');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitPaykeeperModule')) == true) {
            $this->postProcess();
        }

        return $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitPaykeeperModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => $this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'name' => 'PAYKEEPER_URL',
                        'label' => $this->l('URL формы'),
                    ),
                    array(
                        'type' => 'text',
                        'name' => 'PAYKEEPER_SECRET',
                        'label' => $this->l('Секретный ключ'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(            
            'PAYKEEPER_URL' => Configuration::get('PAYKEEPER_URL', null),
            'PAYKEEPER_SECRET' => Configuration::get('PAYKEEPER_SECRET', null),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        $customer = new Customer((int)($params['cart']->id_customer));
        $address  = new Address((int)($params['cart']->id_address_invoice));

        $this->context->controller->addCSS(($this->_path).'hook.css', 'all');

        $this->smarty->assign(array(
            'this_path' => $this->_path
        ));

        $this->setOrderTotal($params['cart']->getOrderTotal());
        $clientid     = $customer->firstname . " " . $customer->lastname;
        $orderid      = $params['cart']->id;
        $client_phone = $address->phone;
        $client_email = $customer->email;
        $service_name = $this->context->customer->secure_key;
        $secret       = Configuration::get('PAYKEEPER_SECRET', null);
        $url          = Configuration::get('PAYKEEPER_URL', null);

        //GENERATE FZ54 CART
        $cart_sum = 0;
        $tax = "none";
        $tax_sum = 0;
        $tax_value = 0;

        $cart_data = $params['cart']->getProducts();

        foreach ($cart_data as $product) {
            $name = $product["name"];
            $price = number_format($product['price_wt'], 2, ".", "");
            $quantity = $product['quantity'];
            $sum = number_format($price*$quantity, 2, ".", "");
            $cart_sum += $sum;

            if ( (int)$product["rate"] != 0) {
                switch((int)$product["rate"]) {
                    case 10:
                        $tax = "vat10";
                        $tax_sum = round((float)(($sum/110)*10), 2);
                        break;
                    case 18:
                        $tax = "vat18";
                        $tax_sum = round((float)(($sum/118)*18), 2);
                        break;
                }
            }

            $this->updateProductCart($name, $price, $quantity, $sum, $tax, $tax_sum);

            $tax = "none";
            $tax_sum = 0;
            $tax_value = 0;
        }

        //add shipping service
        $carrier = new Carrier((int)$params['cart']->id_carrier, $params['cart']->id_lang);
        $this->setShippingPrice($params['cart']->getCarrierCost($params['cart']->id_carrier));
        $shipping_name = $carrier->name;
        $shipping_tax = round((float)(($this->getShippingPrice()/118)*18), 2);

        $this->updateProductCart($shipping_name, $this->getShippingPrice(), 1, $this->getShippingPrice(), "vat18", $shipping_tax);

        //set discounts
        $cart_sum = $this->setDiscounts($params, $cart_sum);
        
        //handle possible precision problem
        $is_correct_precision = $this->isCorrectPrecision($cart_sum);
        
        $product_cart_encoded = json_encode($this->product_cart);

        $to_hash = $this->getOrderTotal()   .
                   $clientid                .
                   $orderid                 .
                   $service_name            .
                   $client_email            .
                   $client_phone            .
                   $secret;
        $sign = hash ('sha256', $to_hash);

        //lang parameter
        $lang = ($this->context->language->iso_code) ?
                 $this->context->language->iso_code :
                 "ru";
        $sb_title = ($lang == "ru") ?
                     "Оплата картами Visa и MasterCard на сайте" :
                     "Pay with Visa and MasterCard online";

        $postdata = array(
            'sum'             => $this->getOrderTotal(),
            'clientid'        => $clientid,
            'orderid'         => $orderid,
            'client_phone'    => $client_phone,
            'client_email'    => $client_email,
            'service_name'    => $service_name,
            'cart'            => $product_cart_encoded,
            'sign'            => $sign,
            'lang'            => $lang,
            'sb_title'        => $sb_title,
            'url'             => $url,
            'precision_error' => $is_correct_precision,
        );

        $this->smarty->assign('postdata', $postdata);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false)
            return;

        $order = $params['objOrder'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR'))
            $this->smarty->assign('status', 'ok');

        $this->smarty->assign(array(
            'id_order' => $order->id,
            'reference' => $order->reference,
            'params' => $params,
            'total' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false),
        ));

        return $this->display(__FILE__, 'views/templates/hook/confirmation.tpl');
    }

#    public function hookDisplayPayment($params)
#    {
#
#        $customer = new Customer((int)($params['cart']->id_customer));
#
#        $payment_parameters = http_build_query(array(
#                    "clientid"=>$customer->email,
#                    "orderid"=>$params['cart']->id,
#                    "sum"=>number_format($params['cart']->getOrderTotal(), 2, '.', ''),
#                    ));
#        $options = array("http"=>array(
#                    "method"=>"POST",
#                    "header"=>
#                    "Content-type: application/x-www-form-urlencoded",
#                    "content"=>$payment_parameters
#                       ));
#        $context = stream_context_create($options);
#         
#        return file_get_contents(Configuration::get('PAYKEEPER_URL', null),FALSE, $context);
#    }
#
    public function hookDisplayPaymentReturn()
    {
        /* Place your code here. */
        echo "Thank you for you order!";
    }

    private function updateProductCart($name="", $price=0, $quantity=0, $sum=0, $tax="none", $tax_value=0)
    {
        //update fz54 cart
        $name = str_replace("\n ", "", $name);
        $name = str_replace("\r ", "", $name);
        $name = str_replace(" ", "&nbsp;", $name);
        $name = preg_replace('/"{1,}/','\'',$name);
        $this->product_cart[] = array(
            "name" => htmlspecialchars($name),
            "price" => $price,
            "quantity" => $quantity,
            "sum" => $sum,
            "tax" => $tax,
            "tax_sum" => number_format($tax_value, 2, ".", "")
        );
    }

    private function getProductCart()
    {
        return $this->product_cart;
    }

    private function setDiscounts($params, $cart_sum)
    {
        //set discounts
        $product_cart_sum = $cart_sum;
        if (!empty($params['cart']->getDiscounts())) {
            $order_sum = $this->getOrderTotal() - $this->getShippingPrice();
            if ($product_cart_sum > $order_sum) {
                $percent = ($product_cart_sum - $order_sum) / ($product_cart_sum/100);
                $product_cart_sum = 0;
                for ($pos=0; $pos<count($this->product_cart)-1; $pos++) {
                    $price_diff = $this->product_cart[$pos]['price']/100*$percent;
                    $this->product_cart[$pos]['price'] -= $price_diff;
                    $this->product_cart[$pos]['price'] = number_format($this->product_cart[$pos]['price'], 2, ".", "");
                    $this->product_cart[$pos]['sum'] = $this->product_cart[$pos]['price']*$this->product_cart[$pos]['quantity'];
                    $this->product_cart[$pos]['sum'] = number_format($this->product_cart[$pos]['sum'], 2, ".", "");
                    $product_cart_sum += $this->product_cart[$pos]['sum'];
                    //recalculate taxes
                    if ( (int)$product['rate'] != 0) {
                        switch((int)$product['rate']) {
                            case "10":
                                $this->product_cart[$pos]['tax_sum'] = round((float)(($this->product_cart[$pos]['sum']/110)*10), 2);
                                break;
                            case "18":
                                $this->product_cart[$pos]['tax_sum'] = round((float)(($this->product_cart[$pos]['sum']/118)*18), 2);
                                break;
                        }
                    }
                }
            }
        }
        return $product_cart_sum;
    }

    private function isCorrectPrecision($cart_sum)
    {
        //handle possible precision problem
        $product_cart_sum = $cart_sum;
        $total_sum = number_format($this->getOrderTotal(), 2, ".", "");
        //add shipping sum to cart sum
        if ($this->getShippingPrice() > 0)
            $product_cart_sum += $this->product_cart[count($this->product_cart)-1]['sum'];
        //DEBUG
        //echo "total: " . $total_sum . " - cart: " . $cart_sum;
        $diff_sum = $product_cart_sum - $total_sum;
        if (abs($diff_sum) <= 0.019) {
            $this->setOrderTotal(number_format($total_sum+$diff_sum, 2, ".", ""));
            return True;
        }
        else return False;
    }

    private function setOrderTotal($value)
    {
        $this->order_total = $value;
    }

    private function getOrderTotal()
    {
        return $this->order_total;
    }

    private function setShippingPrice($value)
    {
        $this->shipping_price = $value;
    }

    private function getShippingPrice()
    {
        return $this->shipping_price;
    }
}
